import json
import os
import re

from descaling.desktop_environment.interface import DesktopEnvironmentInterface
import sh


class Gnome(DesktopEnvironmentInterface):
    name = "gnome"

    @staticmethod
    def is_current_desktop_environment() -> bool:
        if os.environ.get("DESKTOP_SESSION") == "gnome":
            return True

        if os.environ.get("XDG_SESSION_DESKTOP") == "gnome":
            return True

        return False

    @staticmethod
    def pre_xrandr_hook(scaling: float):
        sh.gsettings(
            "set", "org.gnome.desktop.interface",
            "scaling-factor", int(scaling)
        )

        sh.gsettings(
            "set", "org.gnome.settings-daemon.plugins.xsettings",
            "overrides", f"{{'Gdk/WindowScalingFactor': <{int(scaling)}>}}"
        )

    @staticmethod
    def post_xrandr_hook(scaling: float):
        pass

    @staticmethod
    def get_ui_scaling() -> float:
        scaling = float(str(sh.gsettings("get", "org.gnome.desktop.interface", "scaling-factor")).split(" ")[1])

        result = re.findall(
            r"Gdk\/WindowScalingFactor[\'\"]\W*:\W*<(\d*)>",
            str(sh.gsettings("get", "org.gnome.settings-daemon.plugins.xsettings", "overrides"))
        )

        if len(result) > 0:
            scaling = max(scaling, float(result[0]))

        return scaling


import abc
from abc import ABC, abstractmethod


class DesktopEnvironmentInterface(ABC):
    name: str

    @staticmethod
    @abstractmethod
    def is_current_desktop_environment() -> bool:
        pass

    @staticmethod
    @abstractmethod
    def pre_xrandr_hook(scaling: float):
        pass

    @staticmethod
    @abstractmethod
    def post_xrandr_hook(scaling: float):
        pass

    @staticmethod
    @abstractmethod
    def get_ui_scaling() -> float:
        pass



import dataclasses
import platform

import enum
import os
from typing import Optional
import sh


class OperatingSystem(enum.Enum):
    Windows = "windows"
    Linux = "linux"
    Darwin = "darwin"


class SessionType(enum.Enum):
    X11 = "x11"
    Wayland = "wayland"


@dataclasses.dataclass
class Environment:
    operating_system: OperatingSystem
    session_type: Optional[SessionType]


def detect_session_type() -> Optional[SessionType]:
    if "XDG_SESSION_TYPE" in os.environ:
        session = os.environ.get("XDG_SESSION_TYPE")

        if session == "x11":
            return SessionType.X11
        elif session == "wayland":
            return SessionType.Wayland

    session = sh.loginctl(
        "show-session",
        sh.loginctl(
            "show-user",
            sh.whoami().replace("\n", ""),
            p="Display",
            value=True
        ).replace("\n", ""),
        p="Type",
        value=True
    )

    if session == "x11":
        return SessionType.X11
    elif session == "wayland":
        return SessionType.Wayland


def detect_operating_system() -> OperatingSystem:
    operating_system = platform.system()

    if operating_system == "Linux":
        return OperatingSystem.Linux
    elif operating_system == "Windows":
        return OperatingSystem.Windows
    elif operating_system == "Darwin":
        return OperatingSystem.Darwin


from typing import Optional

from .desktop_environment import DesktopEnvironmentInterface, desktop_environment_collection
from .environment import OperatingSystem, detect_operating_system, detect_session_type, SessionType


def get_desktop_environment() -> Optional[DesktopEnvironmentInterface]:
    if (operating_system := detect_operating_system()) != OperatingSystem.Linux:
        raise EnvironmentError(f"This library only supports Linux, {operating_system} was found.")

    if (session_type := detect_session_type()) != SessionType.X11:
        raise EnvironmentError(f"This library only supports x11, {session_type} was found")

    for desktop_environment in desktop_environment_collection:
        if desktop_environment.is_current_desktop_environment():
            return desktop_environment

    raise EnvironmentError(f"The current desktop environment is not supported")



from setuptools import setup, find_packages

setup(
    name='descaling',
    version='1.0.0',
    packages=find_packages(),
    url='',
    license='',
    author='lagmoellertim',
    author_email='mail@lagmoellertim.de',
    description='',
)
